# frozen_string_literal: true

require "dhall"

require "dhall/rails/cache"
require "dhall/rails/coder"
require "dhall/rails/configuration"
require "dhall/rails/encrypted_configuration"
require "dhall/rails/i18n"
require "dhall/rails/application"

module Dhall
	module Rails
		using Dhall::AsDhall

		def self.load(
			source,
			resolver: Resolvers::Default.new(
				environment_reader: method(:environment_reader),
				cache:              Cache.new
			),
			timeout: 10
		)
			Dhall.load(source, resolver: resolver, timeout: timeout).sync
		end

		def self.load_decoded(
			source,
			resolver: Resolvers::Default.new(
				environment_reader: method(:environment_reader),
				cache:              Cache.new
			),
			timeout: 10,
			transform_keys: :to_s
		)
			Coder::SIMPLE.decode(
				load(source, resolver: resolver, timeout: timeout),
				transform_keys: transform_keys
			)
		end

		module PathnameAsDhallText
			refine Pathname do
				def as_dhall
					Text.new(value: directory? ? "#{cleanpath}/" : cleanpath)
				end
			end
		end

		using PathnameAsDhallText

		def self.dig_dhall(root, path)
			path.reduce(root) { |o, m| o.public_send(m) }.as_dhall
		end

		def self.environment_reader(sources)
			sources.map do |source|
				Promise.resolve(nil).then do
					if source.var.start_with?("Rails.")
						dig_dhall(::Rails, source.var.split(/\./)[1..-1])
					else
						ENV.fetch(source)
					end
				end
			end
		end
	end
end
