# frozen_string_literal: true

module Dhall
	module Rails
		class Cache
			def initialize(cache=::Rails.cache)
				@cache = cache
				@ram = Resolvers::RamCache.new
			end

			def fetch(key, &block)
				Promise.resolve(nil).then do
					if key.is_a?(String) && key.start_with?("sha256:")
						unless (in_cache = @cache.read(key)).nil?
							return in_cache
						end

						yield.then { |result| result.tap { @cache.write(key, result) } }
					else
						@ram.fetch(key, &block)
					end
				end
			end
		end
	end
end
